@@ -21,9 +21,10 @@ use crate::{
 use arrow::record_batch::RecordBatch;
 use arrow_array::cast::as_primitive_array;
 use arrow_schema::{DataType, Schema};
+use datafusion::physical_expr_common::physical_expr::down_cast_any_ref;
 use datafusion::physical_plan::ColumnarValue;
 use datafusion_common::{internal_err, Result, ScalarValue};
-use datafusion_physical_expr::{aggregate::utils::down_cast_any_ref, PhysicalExpr};
+use datafusion_physical_expr::PhysicalExpr;
 use std::{
     any::Any,
     fmt::Display,
