@@ -32,6 +32,7 @@ use arrow::{datatypes::*, ipc::writer::StreamWriter};
 use async_trait::async_trait;
 use bytes::Buf;
 use crc32fast::Hasher;
+use datafusion::physical_plan::metrics::Time;
 use datafusion::{
     arrow::{
         array::*,
@@ -239,14 +240,19 @@ impl PartitionBuffer {
     }
 
     /// Appends all rows of given batch into active array builders.
-    fn append_batch(&mut self, batch: &RecordBatch) -> Result<isize> {
+    fn append_batch(&mut self, batch: &RecordBatch, time_metric: &Time) -> Result<isize> {
         let columns = batch.columns();
         let indices = (0..batch.num_rows()).collect::<Vec<usize>>();
-        self.append_rows(columns, &indices)
+        self.append_rows(columns, &indices, time_metric)
     }
 
     /// Appends rows of specified indices from columns into active array builders.
-    fn append_rows(&mut self, columns: &[ArrayRef], indices: &[usize]) -> Result<isize> {
+    fn append_rows(
+        &mut self,
+        columns: &[ArrayRef],
+        indices: &[usize],
+        time_metric: &Time,
+    ) -> Result<isize> {
         let mut mem_diff = 0;
         let mut start = 0;
 
@@ -263,7 +269,10 @@ impl PartitionBuffer {
                 });
             self.num_active_rows += end - start;
             if self.num_active_rows >= self.batch_size {
+                let mut timer = time_metric.timer();
                 mem_diff += self.flush()?;
+                timer.stop();
+
                 mem_diff += self.init_active_if_necessary()?;
             }
             start = end;
@@ -592,6 +601,9 @@ struct ShuffleRepartitionerMetrics {
 
     /// total spilled bytes during the execution of the operator
     spilled_bytes: Count,
+
+    /// The original size of spilled data. Different to `spilled_bytes` because of compression.
+    data_size: Count,
 }
 
 impl ShuffleRepartitionerMetrics {
@@ -600,6 +612,7 @@ impl ShuffleRepartitionerMetrics {
             baseline: BaselineMetrics::new(metrics, partition),
             spill_count: MetricBuilder::new(metrics).spill_count(partition),
             spilled_bytes: MetricBuilder::new(metrics).spilled_bytes(partition),
+            data_size: MetricBuilder::new(metrics).counter("data_size", partition),
         }
     }
 }
@@ -683,7 +696,10 @@ impl ShuffleRepartitioner {
             ));
         }
 
-        let _timer = self.metrics.baseline.elapsed_compute().timer();
+        // Update data size metric
+        self.metrics.data_size.add(input.get_array_memory_size());
+
+        let time_metric = self.metrics.baseline.elapsed_compute();
 
         // NOTE: in shuffle writer exec, the output_rows metrics represents the
         // number of rows those are written to output data file.
@@ -754,8 +770,11 @@ impl ShuffleRepartitioner {
 
                     // If the range of indices is not big enough, just appending the rows into
                     // active array builders instead of directly adding them as a record batch.
-                    mem_diff +=
-                        output.append_rows(input.columns(), &shuffled_partition_ids[start..end])?;
+                    mem_diff += output.append_rows(
+                        input.columns(),
+                        &shuffled_partition_ids[start..end],
+                        time_metric,
+                    )?;
                 }
 
                 if mem_diff > 0 {
@@ -782,7 +801,7 @@ impl ShuffleRepartitioner {
                 );
 
                 let output = &mut buffered_partitions[0];
-                output.append_batch(&input)?;
+                output.append_batch(&input, time_metric)?;
             }
             other => {
                 // this should be unreachable as long as the validation logic
@@ -798,7 +817,6 @@ impl ShuffleRepartitioner {
 
     /// Writes buffered shuffled record batches into Arrow IPC bytes.
     async fn shuffle_write(&mut self) -> Result<SendableRecordBatchStream> {
-        let _timer = self.metrics.baseline.elapsed_compute().timer();
         let num_output_partitions = self.num_output_partitions;
         let mut buffered_partitions = self.buffered_partitions.lock().await;
         let mut output_batches: Vec<Vec<u8>> = vec![vec![]; num_output_partitions];
@@ -823,14 +841,21 @@ impl ShuffleRepartitioner {
             .map_err(|e| DataFusionError::Execution(format!("shuffle write error: {:?}", e)))?;
 
         for i in 0..num_output_partitions {
+            let mut timer = self.metrics.baseline.elapsed_compute().timer();
+
             offsets[i] = output_data.stream_position()?;
             output_data.write_all(&output_batches[i])?;
+
+            timer.stop();
+
             output_batches[i].clear();
 
             // append partition in each spills
             for spill in &output_spills {
                 let length = spill.offsets[i + 1] - spill.offsets[i];
                 if length > 0 {
+                    let mut timer = self.metrics.baseline.elapsed_compute().timer();
+
                     let mut spill_file =
                         BufReader::new(File::open(spill.file.path()).map_err(|e| {
                             DataFusionError::Execution(format!("shuffle write error: {:?}", e))
@@ -839,15 +864,22 @@ impl ShuffleRepartitioner {
                     std::io::copy(&mut spill_file.take(length), &mut output_data).map_err(|e| {
                         DataFusionError::Execution(format!("shuffle write error: {:?}", e))
                     })?;
+
+                    timer.stop();
                 }
             }
         }
+        let mut timer = self.metrics.baseline.elapsed_compute().timer();
         output_data.flush()?;
+        timer.stop();
 
         // add one extra offset at last to ease partition length computation
         offsets[num_output_partitions] = output_data
             .stream_position()
             .map_err(|e| DataFusionError::Execution(format!("shuffle write error: {:?}", e)))?;
+
+        let mut timer = self.metrics.baseline.elapsed_compute().timer();
+
         let mut output_index =
             BufWriter::new(File::create(index_file).map_err(|e| {
                 DataFusionError::Execution(format!("shuffle write error: {:?}", e))
@@ -859,6 +891,8 @@ impl ShuffleRepartitioner {
         }
         output_index.flush()?;
 
+        timer.stop();
+
         let used = self.reservation.size();
         self.reservation.shrink(used);
 
@@ -878,6 +912,10 @@ impl ShuffleRepartitioner {
         self.metrics.spill_count.value()
     }
 
+    fn data_size(&self) -> usize {
+        self.metrics.data_size.value()
+    }
+
     async fn spill(&self) -> Result<usize> {
         log::debug!(
             "ShuffleRepartitioner spilling shuffle data of {} to disk while inserting ({} time(s) so far)",
@@ -891,6 +929,8 @@ impl ShuffleRepartitioner {
             return Ok(0);
         }
 
+        let mut timer = self.metrics.baseline.elapsed_compute().timer();
+
         let spillfile = self
             .runtime
             .disk_manager
@@ -902,6 +942,8 @@ impl ShuffleRepartitioner {
         )
         .await?;
 
+        timer.stop();
+
         let mut spills = self.spills.lock().await;
         let used = self.reservation.size();
         self.metrics.spill_count.add(1);
@@ -955,6 +997,7 @@ impl Debug for ShuffleRepartitioner {
             .field("memory_used", &self.used())
             .field("spilled_bytes", &self.spilled_bytes())
             .field("spilled_count", &self.spill_count())
+            .field("data_size", &self.data_size())
             .finish()
     }
 }
