@@ -25,7 +25,9 @@ import org.apache.spark.SparkConf
 import org.apache.spark.internal.Logging
 import org.apache.spark.network.util.ByteUnit
 import org.apache.spark.sql.{SparkSession, SparkSessionExtensions}
+import org.apache.spark.sql.catalyst.expressions.{Divide, DoubleLiteral, EqualNullSafe, EqualTo, Expression, FloatLiteral, GreaterThan, GreaterThanOrEqual, KnownFloatingPointNormalized, LessThan, LessThanOrEqual, NamedExpression, PlanExpression, Remainder}
 import org.apache.spark.sql.catalyst.expressions.aggregate.{Final, Partial}
+import org.apache.spark.sql.catalyst.optimizer.NormalizeNaNAndZero
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.catalyst.trees.TreeNode
 import org.apache.spark.sql.comet._
@@ -35,15 +37,18 @@ import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.adaptive.{AQEShuffleReadExec, BroadcastQueryStageExec, ShuffleQueryStageExec}
 import org.apache.spark.sql.execution.aggregate.{BaseAggregateExec, HashAggregateExec, ObjectHashAggregateExec}
 import org.apache.spark.sql.execution.datasources._
+import org.apache.spark.sql.execution.datasources.csv.CSVFileFormat
 import org.apache.spark.sql.execution.datasources.json.JsonFileFormat
 import org.apache.spark.sql.execution.datasources.parquet.ParquetFileFormat
 import org.apache.spark.sql.execution.datasources.v2.BatchScanExec
+import org.apache.spark.sql.execution.datasources.v2.csv.CSVScan
 import org.apache.spark.sql.execution.datasources.v2.json.JsonScan
 import org.apache.spark.sql.execution.datasources.v2.parquet.ParquetScan
 import org.apache.spark.sql.execution.exchange.{BroadcastExchangeExec, ReusedExchangeExec, ShuffleExchangeExec}
 import org.apache.spark.sql.execution.joins.{BroadcastHashJoinExec, ShuffledHashJoinExec, SortMergeJoinExec}
 import org.apache.spark.sql.execution.window.WindowExec
 import org.apache.spark.sql.internal.SQLConf
+import org.apache.spark.sql.types.{DoubleType, FloatType}
 
 import org.apache.comet.CometConf._
 import org.apache.comet.CometExplainInfo.getActualPlan
@@ -92,7 +97,17 @@ class CometSparkSessionExtensions
         }
         plan
       } else {
+
+        def isDynamicPruningFilter(e: Expression): Boolean =
+          e.exists(_.isInstanceOf[PlanExpression[_]])
+
         plan.transform {
+          case scanExec: FileSourceScanExec
+              if COMET_DPP_FALLBACK_ENABLED.get() &&
+                scanExec.partitionFilters.exists(isDynamicPruningFilter) =>
+            withInfo(scanExec, "DPP not supported")
+            scanExec
+
           // data source V2
           case scanExec: BatchScanExec
               if scanExec.scan.isInstanceOf[ParquetScan] &&
@@ -827,6 +842,58 @@ class CometSparkSessionExtensions
       }
     }
 
+    def normalizePlan(plan: SparkPlan): SparkPlan = {
+      plan.transformUp {
+        case p: ProjectExec =>
+          val newProjectList = p.projectList.map(normalize(_).asInstanceOf[NamedExpression])
+          ProjectExec(newProjectList, p.child)
+        case f: FilterExec =>
+          val newCondition = normalize(f.condition)
+          FilterExec(newCondition, f.child)
+      }
+    }
+
+    // Spark will normalize NaN and zero for floating point numbers for several cases.
+    // See `NormalizeFloatingNumbers` optimization rule in Spark.
+    // However, one exception is for comparison operators. Spark does not normalize NaN and zero
+    // because they are handled well in Spark (e.g., `SQLOrderingUtil.compareFloats`). But the
+    // comparison functions in arrow-rs do not normalize NaN and zero. So we need to normalize NaN
+    // and zero for comparison operators in Comet.
+    def normalize(expr: Expression): Expression = {
+      expr.transformUp {
+        case EqualTo(left, right) =>
+          EqualTo(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case EqualNullSafe(left, right) =>
+          EqualNullSafe(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case GreaterThan(left, right) =>
+          GreaterThan(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case GreaterThanOrEqual(left, right) =>
+          GreaterThanOrEqual(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case LessThan(left, right) =>
+          LessThan(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case LessThanOrEqual(left, right) =>
+          LessThanOrEqual(normalizeNaNAndZero(left), normalizeNaNAndZero(right))
+        case Divide(left, right, evalMode) =>
+          Divide(left, normalizeNaNAndZero(right), evalMode)
+        case Remainder(left, right, evalMode) =>
+          Remainder(left, normalizeNaNAndZero(right), evalMode)
+      }
+    }
+
+    def normalizeNaNAndZero(expr: Expression): Expression = {
+      expr match {
+        case _: KnownFloatingPointNormalized => expr
+        case FloatLiteral(f) if !f.equals(-0.0f) => expr
+        case DoubleLiteral(d) if !d.equals(-0.0d) => expr
+        case _ =>
+          expr.dataType match {
+            case _: FloatType | _: DoubleType =>
+              KnownFloatingPointNormalized(NormalizeNaNAndZero(expr))
+            case _ => expr
+          }
+      }
+    }
+
     override def apply(plan: SparkPlan): SparkPlan = {
       // DataFusion doesn't have ANSI mode. For now we just disable CometExec if ANSI mode is
       // enabled.
@@ -852,7 +919,7 @@ class CometSparkSessionExtensions
           plan
         }
       } else {
-        var newPlan = transform(plan)
+        var newPlan = transform(normalizePlan(plan))
 
         // if the plan cannot be run fully natively then explain why (when appropriate
         // config is enabled)
@@ -977,6 +1044,16 @@ class CometSparkSessionExtensions
       val eliminatedPlan = plan transformUp {
         case ColumnarToRowExec(sparkToColumnar: CometSparkToColumnarExec) => sparkToColumnar.child
         case CometSparkToColumnarExec(child: CometSparkToColumnarExec) => child
+        // Spark adds `RowToColumnar` under Comet columnar shuffle. But it's redundant as the
+        // shuffle takes row-based input.
+        case s @ CometShuffleExchangeExec(
+              _,
+              RowToColumnarExec(child),
+              _,
+              _,
+              CometColumnarShuffle,
+              _) =>
+          s.withNewChildren(Seq(child))
       }
 
       eliminatedPlan match {
@@ -1111,6 +1188,7 @@ object CometSparkSessionExtensions extends Logging {
         // Convert Spark DS v1 scan to Arrow format
         case scan: FileSourceScanExec =>
           scan.relation.fileFormat match {
+            case _: CSVFileFormat => CometConf.COMET_CONVERT_FROM_CSV_ENABLED.get(conf)
             case _: JsonFileFormat => CometConf.COMET_CONVERT_FROM_JSON_ENABLED.get(conf)
             case _: ParquetFileFormat => CometConf.COMET_CONVERT_FROM_PARQUET_ENABLED.get(conf)
             case _ => isSparkToArrowEnabled(conf, op)
@@ -1118,6 +1196,7 @@ object CometSparkSessionExtensions extends Logging {
         // Convert Spark DS v2 scan to Arrow format
         case scan: BatchScanExec =>
           scan.scan match {
+            case _: CSVScan => CometConf.COMET_CONVERT_FROM_CSV_ENABLED.get(conf)
             case _: JsonScan => CometConf.COMET_CONVERT_FROM_JSON_ENABLED.get(conf)
             case _: ParquetScan => CometConf.COMET_CONVERT_FROM_PARQUET_ENABLED.get(conf)
             case _ => isSparkToArrowEnabled(conf, op)
