@@ -119,19 +119,21 @@ object CometExec {
 
   def getCometIterator(
       inputs: Seq[Iterator[ColumnarBatch]],
+      numOutputCols: Int,
       nativePlan: Operator): CometExecIterator = {
-    getCometIterator(inputs, nativePlan, CometMetricNode(Map.empty))
+    getCometIterator(inputs, numOutputCols, nativePlan, CometMetricNode(Map.empty))
   }
 
   def getCometIterator(
       inputs: Seq[Iterator[ColumnarBatch]],
+      numOutputCols: Int,
       nativePlan: Operator,
       nativeMetrics: CometMetricNode): CometExecIterator = {
     val outputStream = new ByteArrayOutputStream()
     nativePlan.writeTo(outputStream)
     outputStream.close()
     val bytes = outputStream.toByteArray
-    new CometExecIterator(newIterId, inputs, bytes, nativeMetrics)
+    new CometExecIterator(newIterId, inputs, numOutputCols, bytes, nativeMetrics)
   }
 
   /**
@@ -213,8 +215,12 @@ abstract class CometNativeExec extends CometExec {
         val nativeMetrics = CometMetricNode.fromCometPlan(this)
 
         def createCometExecIter(inputs: Seq[Iterator[ColumnarBatch]]): CometExecIterator = {
-          val it =
-            new CometExecIterator(CometExec.newIterId, inputs, serializedPlanCopy, nativeMetrics)
+          val it = new CometExecIterator(
+            CometExec.newIterId,
+            inputs,
+            output.length,
+            serializedPlanCopy,
+            nativeMetrics)
 
           setSubqueries(it.id, this)
 
@@ -428,6 +434,7 @@ case class CometProjectExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometProjectExec =>
+        this.output == other.output &&
         this.projectList == other.projectList &&
         this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
@@ -436,7 +443,7 @@ case class CometProjectExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(projectList, child)
+  override def hashCode(): Int = Objects.hashCode(output, projectList, child)
 
   override protected def outputExpressions: Seq[NamedExpression] = projectList
 }
@@ -462,6 +469,7 @@ case class CometFilterExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometFilterExec =>
+        this.output == other.output &&
         this.condition == other.condition && this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
@@ -469,7 +477,7 @@ case class CometFilterExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(condition, child)
+  override def hashCode(): Int = Objects.hashCode(output, condition, child)
 
   override def verboseStringWithOperatorId(): String = {
     s"""
@@ -501,6 +509,7 @@ case class CometSortExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometSortExec =>
+        this.output == other.output &&
         this.sortOrder == other.sortOrder && this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
@@ -508,7 +517,7 @@ case class CometSortExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(sortOrder, child)
+  override def hashCode(): Int = Objects.hashCode(output, sortOrder, child)
 
   override lazy val metrics: Map[String, SQLMetric] =
     CometMetricNode.baselineMetrics(sparkContext) ++
@@ -539,6 +548,7 @@ case class CometLocalLimitExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometLocalLimitExec =>
+        this.output == other.output &&
         this.limit == other.limit && this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
@@ -546,7 +556,7 @@ case class CometLocalLimitExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(limit: java.lang.Integer, child)
+  override def hashCode(): Int = Objects.hashCode(output, limit: java.lang.Integer, child)
 }
 
 case class CometGlobalLimitExec(
@@ -569,6 +579,7 @@ case class CometGlobalLimitExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometGlobalLimitExec =>
+        this.output == other.output &&
         this.limit == other.limit && this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
@@ -576,7 +587,7 @@ case class CometGlobalLimitExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(limit: java.lang.Integer, child)
+  override def hashCode(): Int = Objects.hashCode(output, limit: java.lang.Integer, child)
 }
 
 case class CometExpandExec(
@@ -599,6 +610,7 @@ case class CometExpandExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometExpandExec =>
+        this.output == other.output &&
         this.projections == other.projections && this.child == other.child &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
@@ -606,7 +618,7 @@ case class CometExpandExec(
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(projections, child)
+  override def hashCode(): Int = Objects.hashCode(output, projections, child)
 
   // TODO: support native Expand metrics
   override lazy val metrics: Map[String, SQLMetric] = Map.empty
@@ -638,12 +650,14 @@ case class CometUnionExec(
 
   override def equals(obj: Any): Boolean = {
     obj match {
-      case other: CometUnionExec => this.children == other.children
+      case other: CometUnionExec =>
+        this.output == other.output &&
+        this.children == other.children
       case _ => false
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(children)
+  override def hashCode(): Int = Objects.hashCode(output, children)
 }
 
 case class CometHashAggregateExec(
@@ -677,6 +691,7 @@ case class CometHashAggregateExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometHashAggregateExec =>
+        this.output == other.output &&
         this.groupingExpressions == other.groupingExpressions &&
         this.aggregateExpressions == other.aggregateExpressions &&
         this.input == other.input &&
@@ -689,7 +704,7 @@ case class CometHashAggregateExec(
   }
 
   override def hashCode(): Int =
-    Objects.hashCode(groupingExpressions, aggregateExpressions, input, mode, child)
+    Objects.hashCode(output, groupingExpressions, aggregateExpressions, input, mode, child)
 
   override protected def outputExpressions: Seq[NamedExpression] = resultExpressions
 }
@@ -729,6 +744,7 @@ case class CometHashJoinExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometHashJoinExec =>
+        this.output == other.output &&
         this.leftKeys == other.leftKeys &&
         this.rightKeys == other.rightKeys &&
         this.condition == other.condition &&
@@ -742,7 +758,7 @@ case class CometHashJoinExec(
   }
 
   override def hashCode(): Int =
-    Objects.hashCode(leftKeys, rightKeys, condition, buildSide, left, right)
+    Objects.hashCode(output, leftKeys, rightKeys, condition, buildSide, left, right)
 
   override lazy val metrics: Map[String, SQLMetric] =
     CometMetricNode.hashJoinMetrics(sparkContext)
@@ -865,6 +881,7 @@ case class CometBroadcastHashJoinExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometBroadcastHashJoinExec =>
+        this.output == other.output &&
         this.leftKeys == other.leftKeys &&
         this.rightKeys == other.rightKeys &&
         this.condition == other.condition &&
@@ -878,7 +895,7 @@ case class CometBroadcastHashJoinExec(
   }
 
   override def hashCode(): Int =
-    Objects.hashCode(leftKeys, rightKeys, condition, buildSide, left, right)
+    Objects.hashCode(output, leftKeys, rightKeys, condition, buildSide, left, right)
 
   override lazy val metrics: Map[String, SQLMetric] =
     CometMetricNode.hashJoinMetrics(sparkContext)
@@ -918,6 +935,7 @@ case class CometSortMergeJoinExec(
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometSortMergeJoinExec =>
+        this.output == other.output &&
         this.leftKeys == other.leftKeys &&
         this.rightKeys == other.rightKeys &&
         this.condition == other.condition &&
@@ -930,17 +948,10 @@ case class CometSortMergeJoinExec(
   }
 
   override def hashCode(): Int =
-    Objects.hashCode(leftKeys, rightKeys, condition, left, right)
+    Objects.hashCode(output, leftKeys, rightKeys, condition, left, right)
 
   override lazy val metrics: Map[String, SQLMetric] =
-    Map(
-      "input_batches" -> SQLMetrics.createMetric(sparkContext, "Number of batches consumed"),
-      "input_rows" -> SQLMetrics.createMetric(sparkContext, "Number of rows consumed"),
-      "output_batches" -> SQLMetrics.createMetric(sparkContext, "Number of batches produced"),
-      "output_rows" -> SQLMetrics.createMetric(sparkContext, "Number of rows produced"),
-      "peak_mem_used" ->
-        SQLMetrics.createSizeMetric(sparkContext, "Peak memory used for buffered data"),
-      "join_time" -> SQLMetrics.createNanoTimingMetric(sparkContext, "Total time for joining"))
+    CometMetricNode.sortMergeJoinMetrics(sparkContext)
 }
 
 case class CometScanWrapper(override val nativeOp: Operator, override val originalPlan: SparkPlan)
