@@ -27,6 +27,7 @@ import scala.util.Random
 
 import org.apache.hadoop.fs.Path
 import org.apache.spark.sql.{CometTestBase, DataFrame, Row}
+import org.apache.spark.sql.catalyst.optimizer.SimplifyExtractValueOps
 import org.apache.spark.sql.comet.CometProjectExec
 import org.apache.spark.sql.execution.{ColumnarToRowExec, InputAdapter, WholeStageCodegenExec}
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
@@ -35,11 +36,27 @@ import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.internal.SQLConf.SESSION_LOCAL_TIMEZONE
 import org.apache.spark.sql.types.{Decimal, DecimalType}
 
-import org.apache.comet.CometSparkSessionExtensions.{isSpark33Plus, isSpark34Plus}
+import org.apache.comet.CometSparkSessionExtensions.{isSpark33Plus, isSpark34Plus, isSpark40Plus}
 
 class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   import testImplicits._
 
+  test("compare true/false to negative zero") {
+    Seq(false, true).foreach { dictionary =>
+      withSQLConf("parquet.enable.dictionary" -> dictionary.toString) {
+        val table = "test"
+        withTable(table) {
+          sql(s"create table $table(col1 boolean, col2 float) using parquet")
+          sql(s"insert into $table values(true, -0.0)")
+          sql(s"insert into $table values(false, -0.0)")
+
+          checkSparkAnswerAndOperator(
+            s"SELECT col1, negative(col2), cast(col1 as float), col1 = negative(col2) FROM $table")
+        }
+      }
+    }
+  }
+
   test("coalesce should return correct datatype") {
     Seq(true, false).foreach { dictionaryEnabled =>
       withTempDir { dir =>
@@ -145,6 +162,100 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
+  test("date_add with int scalars") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      Seq("TINYINT", "SHORT", "INT").foreach { intType =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          withParquetTable(path.toString, "tbl") {
+            checkSparkAnswerAndOperator(f"SELECT _20 + CAST(2 as $intType) from tbl")
+          }
+        }
+      }
+    }
+  }
+
+  test("date_add with scalar overflow") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withTempDir { dir =>
+        val path = new Path(dir.toURI.toString, "test.parquet")
+        makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+        withParquetTable(path.toString, "tbl") {
+          val (sparkErr, cometErr) =
+            checkSparkMaybeThrows(sql(s"SELECT _20 + ${Int.MaxValue} FROM tbl"))
+          if (isSpark40Plus) {
+            assert(sparkErr.get.getMessage.contains("EXPRESSION_DECODING_FAILED"))
+          } else {
+            assert(sparkErr.get.getMessage.contains("integer overflow"))
+          }
+          assert(cometErr.get.getMessage.contains("`NaiveDate + TimeDelta` overflowed"))
+        }
+      }
+    }
+  }
+
+  test("date_add with int arrays") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      Seq("_2", "_3", "_4").foreach { intColumn => // tinyint, short, int columns
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          withParquetTable(path.toString, "tbl") {
+            checkSparkAnswerAndOperator(f"SELECT _20 + $intColumn FROM tbl")
+          }
+        }
+      }
+    }
+  }
+
+  test("date_sub with int scalars") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      Seq("TINYINT", "SHORT", "INT").foreach { intType =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          withParquetTable(path.toString, "tbl") {
+            checkSparkAnswerAndOperator(f"SELECT _20 - CAST(2 as $intType) from tbl")
+          }
+        }
+      }
+    }
+  }
+
+  test("date_sub with scalar overflow") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withTempDir { dir =>
+        val path = new Path(dir.toURI.toString, "test.parquet")
+        makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+        withParquetTable(path.toString, "tbl") {
+          val (sparkErr, cometErr) =
+            checkSparkMaybeThrows(sql(s"SELECT _20 - ${Int.MaxValue} FROM tbl"))
+          if (isSpark40Plus) {
+            assert(sparkErr.get.getMessage.contains("EXPRESSION_DECODING_FAILED"))
+          } else {
+            assert(sparkErr.get.getMessage.contains("integer overflow"))
+          }
+          assert(cometErr.get.getMessage.contains("`NaiveDate - TimeDelta` overflowed"))
+        }
+      }
+    }
+  }
+
+  test("date_sub with int arrays") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      Seq("_2", "_3", "_4").foreach { intColumn => // tinyint, short, int columns
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          withParquetTable(path.toString, "tbl") {
+            checkSparkAnswerAndOperator(f"SELECT _20 - $intColumn FROM tbl")
+          }
+        }
+      }
+    }
+  }
+
   test("dictionary arithmetic") {
     // TODO: test ANSI mode
     withSQLConf(SQLConf.ANSI_ENABLED.key -> "false", "parquet.enable.dictionary" -> "true") {
@@ -297,9 +408,6 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("cast timestamp and timestamp_ntz to string") {
-    // TODO: make the test pass for Spark 3.3
-    assume(isSpark34Plus)
-
     withSQLConf(
       SESSION_LOCAL_TIMEZONE.key -> "Asia/Kathmandu",
       CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true") {
@@ -322,9 +430,6 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("cast timestamp and timestamp_ntz to long, date") {
-    // TODO: make the test pass for Spark 3.3
-    assume(isSpark34Plus)
-
     withSQLConf(
       SESSION_LOCAL_TIMEZONE.key -> "Asia/Kathmandu",
       CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true") {
@@ -844,8 +949,11 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     // Enabling ANSI will cause native engine failure, but as we cannot catch
     // native error now, we cannot test it here.
     withSQLConf(SQLConf.ANSI_ENABLED.key -> "false") {
-      withParquetTable(Seq((1, 0, 1.0, 0.0)), "tbl") {
-        checkSparkAnswerAndOperator("SELECT _1 / _2, _3 / _4 FROM tbl")
+      withParquetTable(Seq((1, 0, 1.0, 0.0, -0.0)), "tbl") {
+        checkSparkAnswerAndOperator("SELECT _1 / _2, _3 / _4, _3 / _5 FROM tbl")
+        checkSparkAnswerAndOperator("SELECT _1 % _2, _3 % _4, _3 % _5 FROM tbl")
+        checkSparkAnswerAndOperator("SELECT _1 / 0, _3 / 0.0, _3 / -0.0 FROM tbl")
+        checkSparkAnswerAndOperator("SELECT _1 % 0, _3 % 0.0, _3 % -0.0 FROM tbl")
       }
     }
   }
@@ -1706,6 +1814,7 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
       SQLConf.COALESCE_PARTITIONS_ENABLED.key -> "true",
       CometConf.COMET_ENABLED.key -> "true",
       CometConf.COMET_EXEC_ENABLED.key -> "true",
+      CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "false",
       EXTENDED_EXPLAIN_PROVIDERS_KEY -> "org.apache.comet.ExtendedExplainInfo") {
       val table = "test"
       withTable(table) {
@@ -1953,6 +2062,79 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
+  test("to_json") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withParquetTable(
+        (0 until 100).map(i => {
+          val str = if (i % 2 == 0) {
+            "even"
+          } else {
+            "odd"
+          }
+          (i.toByte, i.toShort, i, i.toLong, i * 1.2f, -i * 1.2d, str, i.toString)
+        }),
+        "tbl",
+        withDictionary = dictionaryEnabled) {
+
+        val fields = Range(1, 8).map(n => s"'col$n', _$n").mkString(", ")
+
+        checkSparkAnswerAndOperator(s"SELECT to_json(named_struct($fields)) FROM tbl")
+        checkSparkAnswerAndOperator(
+          s"SELECT to_json(named_struct('nested', named_struct($fields))) FROM tbl")
+      }
+    }
+  }
+
+  test("to_json escaping of field names and string values") {
+    val gen = new DataGenerator(new Random(42))
+    val chars = "\\'\"abc\t\r\n\f\b"
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withParquetTable(
+        (0 until 100).map(i => {
+          val str1 = gen.generateString(chars, 8)
+          val str2 = gen.generateString(chars, 8)
+          (i.toString, str1, str2)
+        }),
+        "tbl",
+        withDictionary = dictionaryEnabled) {
+
+        val fields = Range(1, 3)
+          .map(n => {
+            val columnName = s"""column "$n""""
+            s"'$columnName', _$n"
+          })
+          .mkString(", ")
+
+        checkSparkAnswerAndOperator(
+          """SELECT 'column "1"' x, """ +
+            s"to_json(named_struct($fields)) FROM tbl ORDER BY x")
+      }
+    }
+  }
+
+  test("to_json unicode") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withParquetTable(
+        (0 until 100).map(i => {
+          (i.toString, "\uD83E\uDD11", "\u018F")
+        }),
+        "tbl",
+        withDictionary = dictionaryEnabled) {
+
+        val fields = Range(1, 3)
+          .map(n => {
+            val columnName = s"""column "$n""""
+            s"'$columnName', _$n"
+          })
+          .mkString(", ")
+
+        checkSparkAnswerAndOperator(
+          """SELECT 'column "1"' x, """ +
+            s"to_json(named_struct($fields)) FROM tbl ORDER BY x")
+      }
+    }
+  }
+
   test("struct and named_struct with dictionary") {
     Seq(true, false).foreach { dictionaryEnabled =>
       withParquetTable(
@@ -2006,6 +2188,87 @@ class CometExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         val df = spark.read.parquet(path.toString)
         checkSparkAnswerAndOperator(df.select(array(col("_2"), col("_3"), col("_4"))))
         checkSparkAnswerAndOperator(df.select(array(col("_4"), col("_11"), lit(null))))
+        checkSparkAnswerAndOperator(
+          df.select(array(array(col("_4")), array(col("_4"), lit(null)))))
+        checkSparkAnswerAndOperator(df.select(array(col("_8"), col("_13"))))
+        // This ends up returning empty strings instead of nulls for the last element
+        // Fixed by https://github.com/apache/datafusion/commit/27304239ef79b50a443320791755bf74eed4a85d
+        // checkSparkAnswerAndOperator(df.select(array(col("_8"), col("_13"), lit(null))))
+        checkSparkAnswerAndOperator(df.select(array(array(col("_8")), array(col("_13")))))
+        checkSparkAnswerAndOperator(df.select(array(col("_8"), col("_8"), lit(null))))
+        checkSparkAnswerAndOperator(df.select(array(struct("_4"), struct("_4"))))
+      // Fixed by https://github.com/apache/datafusion/commit/140f7cec78febd73d3db537a816badaaf567530a
+      // checkSparkAnswerAndOperator(df.select(array(struct(col("_8").alias("a")), struct(col("_13").alias("a")))))
+      }
+    }
+  }
+
+  test("ListExtract") {
+    def assertBothThrow(df: DataFrame): Unit = {
+      checkSparkMaybeThrows(df) match {
+        case (Some(_), Some(_)) => ()
+        case (spark, comet) =>
+          fail(
+            s"Expected Spark and Comet to throw exception, but got\nSpark: $spark\nComet: $comet")
+      }
+    }
+
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withTempDir { dir =>
+        val path = new Path(dir.toURI.toString, "test.parquet")
+        makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 100)
+
+        Seq(true, false).foreach { ansiEnabled =>
+          withSQLConf(
+            CometConf.COMET_ANSI_MODE_ENABLED.key -> "true",
+            SQLConf.ANSI_ENABLED.key -> ansiEnabled.toString(),
+            // Prevent the optimizer from collapsing an extract value of a create array
+            SQLConf.OPTIMIZER_EXCLUDED_RULES.key -> SimplifyExtractValueOps.ruleName) {
+            val df = spark.read.parquet(path.toString)
+
+            val stringArray = df.select(array(col("_8"), col("_8"), lit(null)).alias("arr"))
+            checkSparkAnswerAndOperator(
+              stringArray
+                .select(col("arr").getItem(0), col("arr").getItem(1), col("arr").getItem(2)))
+
+            checkSparkAnswerAndOperator(
+              stringArray.select(
+                element_at(col("arr"), -3),
+                element_at(col("arr"), -2),
+                element_at(col("arr"), -1),
+                element_at(col("arr"), 1),
+                element_at(col("arr"), 2),
+                element_at(col("arr"), 3)))
+
+            // 0 is an invalid index for element_at
+            assertBothThrow(stringArray.select(element_at(col("arr"), 0)))
+
+            if (ansiEnabled) {
+              assertBothThrow(stringArray.select(col("arr").getItem(-1)))
+              assertBothThrow(stringArray.select(col("arr").getItem(3)))
+              assertBothThrow(stringArray.select(element_at(col("arr"), -4)))
+              assertBothThrow(stringArray.select(element_at(col("arr"), 4)))
+            } else {
+              checkSparkAnswerAndOperator(stringArray.select(col("arr").getItem(-1)))
+              checkSparkAnswerAndOperator(stringArray.select(col("arr").getItem(3)))
+              checkSparkAnswerAndOperator(stringArray.select(element_at(col("arr"), -4)))
+              checkSparkAnswerAndOperator(stringArray.select(element_at(col("arr"), 4)))
+            }
+
+            val intArray =
+              df.select(when(col("_4").isNotNull, array(col("_4"), col("_4"))).alias("arr"))
+            checkSparkAnswerAndOperator(
+              intArray
+                .select(col("arr").getItem(0), col("arr").getItem(1)))
+
+            checkSparkAnswerAndOperator(
+              intArray.select(
+                element_at(col("arr"), 1),
+                element_at(col("arr"), 2),
+                element_at(col("arr"), -1),
+                element_at(col("arr"), -2)))
+          }
+        }
       }
     }
   }
